import { Plugin, PluginKey, Selection } from 'prosemirror-state';
import { Node } from '@kerebron/editor';
import { setBlockType, } from '@kerebron/editor/commands';
import { textblockTypeInputRule, } from '@kerebron/editor/plugins/input-rules';
import { codeMirrorBlockNodeView } from './codeMirrorBlockNodeView.js';
import { defaultSettings } from './defaults.js';
import languageLoaders, { legacyLanguageLoaders } from './languageLoaders.js';
import { dracula } from 'thememirror';
import { createNodeFromObject } from '@kerebron/editor/utilities';
export const codeMirrorBlockKey = new PluginKey('codemirror-block');
function arrowHandler(dir) {
    return (state, dispatch, view) => {
        if (state.selection.empty && view.endOfTextblock(dir)) {
            let side = dir == 'left' || dir == 'up' ? -1 : 1;
            let $head = state.selection.$head;
            let nextPos = Selection.near(state.doc.resolve(side > 0 ? $head.after() : $head.before()), side);
            if (nextPos.$head && nextPos.$head.parent.type.name == 'code_block') {
                dispatch(state.tr.setSelection(nextPos));
                return true;
            }
        }
        return false;
    };
}
export class NodeCodeMirror extends Node {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "name", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 'code_block'
        });
        // requires = ['doc'];
        Object.defineProperty(this, "automerge", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {
                block: this.name,
                attrParsers: {
                    fromAutomerge: (block) => ({ lang: block.attrs.lang }),
                    fromProsemirror: (node) => ({ lang: node.attrs.lang }),
                },
            }
        });
    }
    getConverters(editor, schema) {
        return {
            'text/code-only': {
                fromDoc(document) {
                    const retVal = [];
                    if (document.content) {
                        for (const node of document.content.toJSON()) {
                            if ('code_block' === node.type && Array.isArray(node.content)) {
                                for (const content of node.content) {
                                    retVal.push(content.text);
                                }
                            }
                        }
                    }
                    return retVal.join('');
                },
                toDoc(code) {
                    const content = {
                        'type': 'doc_code',
                        'content': [
                            {
                                'type': 'code_block',
                                'attrs': {
                                    'lang': schema.topNodeType.spec.defaultAttrs?.lang,
                                },
                                'content': [
                                    {
                                        'type': 'text',
                                        'text': code,
                                    },
                                ],
                            },
                        ],
                    };
                    return createNodeFromObject(content, schema, {
                        slice: false,
                        errorOnInvalidContent: false,
                    });
                },
            },
        };
    }
    getNodeSpec() {
        return {
            content: 'text*',
            marks: '',
            group: 'block',
            code: true,
            defining: true,
            parseDOM: [
                {
                    tag: 'pre',
                    preserveWhitespace: 'full',
                    getAttrs(dom) {
                        return {
                            lang: dom.getAttribute('lang'),
                        };
                    },
                },
            ],
            attrs: { lang: { default: null } },
            toDOM(node) {
                const { lang } = node.attrs;
                return ['pre', { lang }, ['code', 0]];
            },
        };
    }
    getInputRules(type) {
        /// Given a code block node type, returns an input rule that turns a
        /// textblock starting with three backticks into a code block.
        return [
            textblockTypeInputRule(/^```$/, type),
        ];
    }
    getCommands(editor, type) {
        return {
            'setCodeBlock': (lang) => setBlockType(type, { lang }),
            ArrowLeft: () => arrowHandler('left'),
            ArrowRight: () => arrowHandler('right'),
            ArrowUp: () => arrowHandler('up'),
            ArrowDown: () => arrowHandler('down'),
        };
    }
    getKeyboardShortcuts() {
        return {
            'Shift-Ctrl-"': 'setCodeBlock',
            'ArrowLeft': 'ArrowLeft',
            'ArrowRight': 'ArrowRight',
            'ArrowUp': 'ArrowUp',
            'ArrowDown': 'ArrowDown',
        };
    }
    getProseMirrorPlugins(editor) {
        const codeMirrorBlockPlugin = (settings) => new Plugin({
            key: codeMirrorBlockKey,
            props: {
                nodeViews: {
                    [this.name]: codeMirrorBlockNodeView(settings),
                },
            },
        });
        const LANGS = [
            'diff',
            'dockerfile',
            'http',
            'nginx',
            'properties',
            'shell',
            'toml',
            'yaml',
            'sql',
            'javascript',
            'cpp',
            'css',
            'xml',
            'java',
            'json',
            'markdown',
            'rust',
            'html',
        ];
        return [
            codeMirrorBlockPlugin({
                provider: this.config.provider,
                languageWhitelist: this.config.languageWhitelist || LANGS,
                ...defaultSettings,
                readOnly: this.config.readOnly,
                languageLoaders: { ...languageLoaders, ...legacyLanguageLoaders },
                undo: () => {
                    editor.chain().undo().run();
                },
                redo: () => {
                    editor.chain().redo().run();
                },
                theme: [dracula],
            }),
            // keymap(codeBlockKeymap),
        ];
    }
}
